//#include "stdafx.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include <limits>
//#include <stdint.h>
#include <vector>
#include <pcl/console/parse.h>
#include <pcl/io/pcd_io.h>
#include <pcl/point_types.h>
#include <pcl/io/ply_io.h>

#include <pcl/io/obj_io.h>
#include <pcl/io/vtk_lib_io.h>
#include <pcl/PolygonMesh.h>

using namespace std;

int
main(int argc, char** argv)
{

	pcl::PointXYZRGB p;
	std::vector<double> depth;

	int r, g, b;
	double ref;
	ifstream ins;
	ofstream ous;
	bool ispts = false;
	int skipline = 9;

	double maxdepth =-FLT_MAX;
	double mindepth = FLT_MAX;

	bool display_help = false;
	display_help = pcl::console::find_switch(argc, argv, "--help");

	if (display_help || argc <= 1)
	{
		std::cout << "Usage: ptxrgb2png.exe [OPTION]... [ptx]" << std::endl;
		std::cout << "     --help   display this help and exit" << std::endl;

		return (0);
	}

	std::vector<int> pcd_indices;
	pcd_indices = pcl::console::parse_file_extension_argument(argc, argv, ".ptx");

	if (pcd_indices.size() < 1) {
		pcd_indices = pcl::console::parse_file_extension_argument(argc, argv, ".pts");
		if (pcd_indices.size() >= 1) {
			skipline = 8;
			ispts = true;
		}
	}

	if (pcd_indices.size() < 1) {
		std::cout << "ptx file must be specified" << std::endl;
		return (0);
	}

	// ********************* t@C̓ǂݍ *********************
	std::string infile = argv[pcd_indices[0]];
	std::string outfile = argv[pcd_indices[0]];

	outfile.replace(outfile.length() - 3, 3, "pgm");

	ins.open(infile.c_str(), ios::in);

	if(!ins)
	{
		cerr << "Error: cannot open file(" << infile << ")" << endl;
		exit(1);
	}
	else
	{
		int n = 0;
		int width = 0;
		int height = 0;
		string buf;
		getline(ins, buf);
		stringstream ss{ buf };
		string s;
		getline(ss, s, ' ');
		width  = atoi(s.c_str());
		getline(ss, s, ' ');
		height = atoi(s.c_str());
		n++;

		while (ins && getline(ins, buf)) if (n++ >= skipline) break;

		n = 0;
		while (!ins.eof())
		{
			double d;
			ins >> p.x;
			ins >> p.y;
			ins >> p.z;
			ins >> ref;
			if (ispts) {
				r = g = b = 0;
			}
			else {
				ins >> r;
				ins >> g;
				ins >> b;
			}
			p.r = (uint8_t)r;
			p.g = (uint8_t)g;
			p.b = (uint8_t)b;
			d = sqrt(p.x*p.x + p.y*p.y + p.z*p.z);
			if (maxdepth < d) maxdepth = d;
			if (mindepth > d) mindepth = d;
			depth.push_back(d);
			n++;
		}
		ins.close();

		ous.open(outfile.c_str(), ios::binary | ios::out);
		if (!ous)
		{
			cout << "Error: cannot open file(" << outfile << ")" << endl;
			exit(1);
		} 
		else
		{
			ous << "P5" << endl;
			ous << width << " " << height << endl;
			ous << "255" << endl;
			for (int i = 0; i < depth.size(); i++) {
				int h = (int)(i / width);
				int w = (int)(i % width);
				h = height - h - 1;
				if (h <0 || h >= height || w < 0 || w >= width) continue;
				unsigned char d = (unsigned char)(256.0 * (depth[h + w * height] - mindepth) / (maxdepth - mindepth));
				ous.write(reinterpret_cast<char *>(&d), sizeof(d));
			}
		}
		ous.close();

	}
	return 0;
} 
